C$Procedure      PLBORE_2 ( Plates in View after Offset Rotation )
 
      SUBROUTINE PLBORE_2 ( INSTID, OFFROT, ET, PLID, BODY, FOUND )
 
C$ Abstract
C
C     Determine the idcode of the plate that lies along the
C     boresight vector from an instrument after an offset
C     rotation in the instrument's reference frame.
C
C$ Copyright
C
C     Copyright (1999), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C      None.
C
C$ Keywords
C
C      SHAPES
C
C$ Declarations
 
      IMPLICIT NONE
 
      INTEGER               INSTID
      DOUBLE PRECISION      ET
      INTEGER               PLID
      INTEGER               BODY
      LOGICAL               FOUND
      DOUBLE PRECISION      OFFROT   ( 3, 3 )
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     INSTID     I   id-code for some instrument
C     OFFROT     I   offset rotation matrix
C     ET         I   epoch at which to locate plates in the FOV
C     PLID       O   ID code of the plate that lies along the boresight
C     BODY       O   body ID of the body that the plate belongs to
C     FOUND      O   TRUE if there is a plate, FALSE otherwise.
C
C$ Detailed_Input
C
C     INSTID     is the id-code for some instrument. The calling
C                program must load and instrument kernel for this
C                instrument specifying the field of view of the
C                instrument and its orientation relative to some
C                reference frame.
C
C     OFFROT     offset rotation to be applied in the instrument frame
C
C     ET         is an epoch expressed in ephemeris seconds past
C                the J2000 epoch (TDB) at which the user wishes
C                to determine a collection of plates in the field
C                of view of the specified instrument.
C
C$ Detailed_Output
C
C     PLID       is the plate ID of the first plate that is intersected
C                by the boresight of the instrument specified by INSTID.
C                In the event that the boresight intersects the surface
C                model at an edge or vertex, the plate whose normal is
C                closest to anti-parallel to the boresight direction is
C                selected.  If this criteria does not identify uniquely
C                the plate to chose, the plate whose center is closest
C                to the instrument is chose.  If this still does not
C                resolve which plate to chose, then the
C                plate with the smallest ID value (among those in
C                contention) is returned.
C
C     BODY       is the NAIF ID code for the body that the boresight
C                first intersects and to which the plate specified by
C                PLID belongs.
C
C     FOUND      is a logical flag that indicates whether or not
C                the boresight vector does indeed intersect.  If
C                the boresight intersects a plate FOUND is TRUE.
C                Otherwise FOUND is FALSE.
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     1) If a required kernel (see Particulars) below cannot be located
C        an error will be diagnosed by one of the routines called
C        by this routine.
C
C$ Particulars
C
C     Given a set of objects whose surfaces are modeled by
C     a set of covering triangular plates, this routine returns
C     the first plate in the list of plates that is intersected
C     by the boresight of the specified instrument.
C
C     A substantial amount of preparation is required in the
C     users program before this routine can be called.
C
C     The user must "load"
C
C        1) Instrument kernels for the instruments of interest
C        2) C-kernels that relate the orientation of the instrument
C           mounting platform to some other reference frame
C        3) shape models for all objects of interest
C        4) spk files for: the objects on which the observing
C           instruments are mounted; the objects with loaded
C           shape models; the sun.
C        5) PCK files that give the orientation of each of the
C           objects whose shapes are modeled by triangular tilings
C        6) An SCLK kernel
C        7) a "frames" kernel that connects the bodyfixed frames
C           of the various modeled surfaces to the appropriate
C           PCK objects.
C
C     Once these preparations have been performed this routine will
C     return the plate first intersected by the boresight (if one
C     exists.
C
C$ Examples
C
C     When time permits.
C
C$ Restrictions
C
C     None.
C
C$ Author_and_Institution
C
C     J.A. Bytof      (JPL)
C     W.L. Taber      (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    Beta Version 2.0.0, 01-FEB-1999 (JAB)
C
C        Corrected the C$Procedure line.  Deleted unnecessary
C        declarations and now declare TEST to be INTEGER instead
C        of DOUBLE PRECISION.
C
C-    Beta Version 1.1.0, 03-SEP-1998 (WLT)
C
C        Irina Mikheeva of LPL at the University of Arizona identified
C        an error in the the routine PLINVU that was repeated in this
C        routine. In the previous version of this routine, the
C        transformation from the instrument frame to the bodyfixed
C        frame was not performed correctly in the case that the frame
C        associated with the user specified BODY could be identified.
C        The code has been updated so that the computation of this
C        frame in different branches of the routine are now
C        equivalent.
C
C        In particular in the code below, the three lines
C
C           CALL SXFORM ( FRAME,  'J2000', ET,      FRM2J2 )
C           CALL SXFORM ( 'J2000', FRNAME, ET-LT,   TSIPM  )
C           CALL MXMG   ( TSIPM,   FRM2J2, 6, 6, 6, XFORM )
C
C        have replaced the erroneous line
C
C           CALL SXFORM ( FRAME, FRNAME, ET-LT, XFORM )
C
C        of the previous edition of the routine.
C
C-    SPICELIB Version 1.0.0, 28-JUL-1998 (JAB)
C
C        Based on PLBORE, added rotation offset feature.
C
C-&
 
C$ Index_Entries
C
C     plates in the boresight of an instrument.
C
C-&
C
C     SPICELIB Functions
C
 
      DOUBLE PRECISION      VDOT
      DOUBLE PRECISION      VNORM
 
      LOGICAL               RETURN
      LOGICAL               FAILED
 
C
C     Parameters
C
 
      INTEGER               FVROOM
      PARAMETER           ( FVROOM = 40 )
 
      INTEGER               WDSIZE
      PARAMETER           ( WDSIZE = 32 )
 
C
C     Local Variables
C
 
      CHARACTER*(WDSIZE)    FRAME
      CHARACTER*(WDSIZE)    FRNAME
      CHARACTER*(WDSIZE)    SHAPE
 
      DOUBLE PRECISION      BDSITE ( 3 )
      DOUBLE PRECISION      BSIGHT ( 3 )
      DOUBLE PRECISION      CENTR  ( 3 )
      DOUBLE PRECISION      COSANG
      DOUBLE PRECISION      DIST
      DOUBLE PRECISION      EDGE   ( 3, FVROOM )
      DOUBLE PRECISION      EDGES  ( 3, 3      )
      DOUBLE PRECISION      FRM2J2 ( 6, 6 )
      DOUBLE PRECISION      LT
      DOUBLE PRECISION      NEAR
      DOUBLE PRECISION      NORML  ( 3 )
      DOUBLE PRECISION      OBSMAT ( 3, 3 )
      DOUBLE PRECISION      POINTS ( 3, 3 )
      DOUBLE PRECISION      POSOBS ( 3 )
      DOUBLE PRECISION      ROT    ( 3, 3 )
      DOUBLE PRECISION      SCALE
      DOUBLE PRECISION      STATE  ( 6 )
      DOUBLE PRECISION      STEMP  ( 6 )
      DOUBLE PRECISION      TEMP
      DOUBLE PRECISION      TOCENT ( 3 )
      DOUBLE PRECISION      TSIPM  ( 6, 6 )
      DOUBLE PRECISION      XFORM  ( 6, 6 )
 
 
      INTEGER               BCOUNT
      INTEGER               BODEX
      INTEGER               CBOD
      INTEGER               FOVSIZ
      INTEGER               FRCODE
      INTEGER               FSTIDX
      INTEGER               I
      INTEGER               J
      INTEGER               LSTIDX
      INTEGER               NPLATE
      INTEGER               NVERT
      INTEGER               OBS
      INTEGER               PRANGE ( 2 )
      INTEGER               TEMP1
      INTEGER               TEMP2
      INTEGER               TEST
      INTEGER               TMP
      INTEGER               VRANGE ( 2 )
 
      LOGICAL               HITS
      LOGICAL               HAVE
      LOGICAL               KNOWN
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN() ) THEN
         RETURN
      END IF
 
      CALL CHKIN ( 'PLBORE_2' )
 
C
C     Until we have better knowledge we assume there is no intersection
C
      BODY  =  0
      PLID  =  0
      FOUND = .FALSE.
      HAVE  = .FALSE.
 
      NEAR = 9.D99
 
C
C     Get the observer ID associated with this instrument
C
      OBS = INSTID / 1000
C
C     Determine where the boresight and edge of the field of
C     view are pointing.  Transform the boresight vector into a
C     unit vector, so that when get SCALE back from INSANG
C     later in this routine, SCALE will be the distance to the
C     intersept point.
C
      CALL GETFOV ( INSTID, FVROOM, SHAPE, FRAME, BSIGHT, FOVSIZ, EDGE )
      CALL VHAT   ( BSIGHT, BSIGHT )
 
C
C     Apply the offset rotation to the instrument boresight vector.
C
 
      CALL MXV ( OFFROT, BSIGHT, BSIGHT )
 
 
C
C     Determine the number of bodies that have plates
C     models loaded.
C
      CALL PLTOT ( BCOUNT, TEMP1, TEMP2 )
 
C
C     For each body, check the coverage with the field of view.
C
      DO BODEX = 1, BCOUNT
C
C        So far we haven't done any of the initialization work
C        for BOUNDS for this body.  Look up the body identifier
C        from the index
C
         CALL DX2BID ( BODEX, CBOD )
 
C
C        See if we can discern the frame name associated with this
C        body.
C
         CALL CIDFRM ( CBOD, FRCODE, FRNAME, KNOWN )
 
         IF ( KNOWN ) THEN
C
C           We can do this the easy (and less error prone way). We
C           need the state of the observer in the apparent bodyfixed
C           frame of the body.  We also need the state of the sun
C           in the apparent body fixed frame relative to the body
C           corrected for light time to the body from the observer.
C
            CALL SPKEZ ( CBOD,  ET, FRNAME, 'LT+S', OBS,  STATE, LT )
            CALL VMINUS( STATE, POSOBS )
C
C           Get the transformation that transforms vectors in FRAME
C           to the bodyfixed frame of this body.
C
 
            CALL SXFORM ( FRAME,  'J2000', ET,      FRM2J2 )
            CALL SXFORM ( 'J2000', FRNAME, ET-LT,   TSIPM  )
            CALL MXMG   ( TSIPM,   FRM2J2, 6, 6, 6, XFORM  )
 
         ELSE
C
C           We don't have the bodyfixed frame identification loaded.
C           We'll try to do this the old-fashioned way. First get the
C           state of the body relative to the observer and the state
C           of the sun relative to the body in the J2000 frame
C           with appropriate corrections made for light time.
C
            CALL SPKEZ( CBOD, ET,    'J2000', 'LT+S', OBS,  STATE, LT  )
C
C           Construct the transformation from the FRAME of the
C           instrument to the bodyfixed frame.
C
            CALL SXFORM ( FRAME,  'J2000', ET,      FRM2J2 )
            CALL TISBOD ( 'J2000', CBOD,   ET-LT,   TSIPM  )
            CALL MXMG   ( TSIPM,   FRM2J2, 6, 6, 6, XFORM )
C
C           Transform the state of the sun into the bodyfixed frame.
C           And get the apparent state of the observer in the
C           bodyfixed frame.
C
            CALL MXVG   ( TSIPM, STATE, 6, 6, STEMP )
            CALL VMINUS ( STEMP, POSOBS )
 
         END IF
C
C        If something went wrong up above there is no point in
C        going on from here.
C
         IF ( FAILED() ) THEN
            CALL CHKOUT ( 'PLBORE_2' )
            RETURN
         END IF
C
C        We are going to need to subtract the location of the observer
C        from vertices of a plate.  To speed things up a tiny bit,
C        we'll make 3 copies of the observer's location so that
C        we make a single subroutine call to handle the 3 subtractions.
C
         CALL VEQU ( POSOBS, OBSMAT(1,1) )
         CALL VEQU ( POSOBS, OBSMAT(1,2) )
         CALL VEQU ( POSOBS, OBSMAT(1,3) )
 
C
C        Get the rotation component of the frame transformation
C
         DO I = 1, 3
            DO J = 1, 3
               ROT (I,J) = XFORM(I,J)
            END DO
         END DO
C
C        Rotate all of the boresight into the bodyfixed frame. Recall
C        that we already have BSIGHT as a unit vector.  Rotations
C        preserve length so BDSITE will be a unit vector as well.
C
         CALL MXV ( ROT,   BSIGHT, BDSITE )
         CALL VHAT( BDSITE,        BDSITE )
 
C
C        Now examine each plate for this body.
C
         CALL PLTBND ( BODEX, NPLATE, PRANGE, NVERT, VRANGE )
 
         FSTIDX = PRANGE(1)
         LSTIDX = PRANGE(2)
 
         DO J = FSTIDX, LSTIDX
 
C
C           Get the J'th plate center and normal.
C
            CALL NTHPLC ( J, CENTR )
            CALL NTHPLN ( J, NORML )
C
C           Compute the position of the observer relative to the
C           center of the plate.
C
            CALL VSUB ( POSOBS, CENTR, TOCENT )
C
C           Only if this plate is visible do we do anything else
C           with this plate.
C
            IF ( VDOT ( TOCENT, NORML ) .GT. 0.0D0 ) THEN
C
C
C              Retrieve the bodyfixed coordinates of the points that
C              are the vertices of the J'th plate. Compute the edges
C              of the tetrahedral angle having the observer as the apex
C              and the vertices as members of the edge rays.  Finally
C              see if the boresight lies inside the tetrahedral angle.
C
               CALL NTHPLV ( J,          POINTS            )
               CALL VSUBG  ( POINTS,     OBSMAT, 9,  EDGES )
               CALL INSANG ( BDSITE,
     .                       EDGES(1,1), EDGES(1,2), EDGES(1,3),
     .                       HITS,       SCALE )
 
C
C              If the boresight intersects this plate, see if we already
C              had an intersection.  If so see if this plate is closer
C              than the last intersection we had.
C
               IF ( HITS ) THEN
 
                  IF ( HAVE ) THEN
C
C                    If this intersection point is closer than
C                    the last one, this plate must be in front.
C
                     IF      ( SCALE .LT. NEAR ) THEN
 
                        BODY   = CBOD
                        NEAR   = SCALE
                        COSANG = VDOT ( BDSITE, NORML )
 
                        CALL NTHPLI ( J, TMP, PLID )
 
                     ELSE IF ( SCALE .EQ. NEAR ) THEN
C
C                       In this case we must have an edge point.
C                       We pick the plate that whose normal direction
C                       is closest to anti-parallel to the boresight
C                       direction.
C
                        CALL VHAT ( NORML, NORML )
 
                        TEMP = VDOT ( BDSITE, NORML )
 
                        IF ( TEMP .LT. COSANG ) THEN
 
                           COSANG = TEMP
                           BODY   = CBOD
                           DIST   = VNORM ( TOCENT )
 
                           CALL NTHPLI ( J, TMP, PLID )
 
                        ELSE IF ( TEMP .EQ. COSANG ) THEN
C
C                          Hmmmm.  We've got two parallel plates that
C                          share an edge or vertex.  Pick the one
C                          with the closest center.
C
                           TEMP = VNORM ( TOCENT )
 
                           IF      ( TEMP .LT. DIST ) THEN
 
                              DIST = TEMP
                              CALL NTHPLI ( J, TMP, PLID )
 
                           ELSE IF ( TEMP .EQ. DIST ) THEN
C
C                             The centers of the plates are equidistant.
C                             That's the end of the geometric tests.
C                             We just take the one that had the lower
C                             plate id number.
C
                              CALL NTHPLI ( J, TMP, TEST )
 
                              IF ( TEST .LT. PLID ) THEN
                                 PLID = TEST
                                 BODY = CBOD
                              END IF
 
                           END IF
 
                        END IF
 
                     END IF
 
                  ELSE
 
                     HAVE   = .TRUE.
                     NEAR   = SCALE
                     COSANG = VDOT ( BDSITE, NORML )
                     DIST   = VNORM( TOCENT )
                     BODY   = CBOD
 
                     CALL NTHPLI ( J, TMP, PLID )
 
                  END IF
 
               END IF
 
            END IF
C
C           Fetch the next plate for this body.
C
         END DO
 
C
C        We are done with the current body.  Do the same set of
C        steps for the next body in the list of bodies that have
C        plate models stored in the plate manager.
C
 
      END DO
 
C
C     We've done all of the tedious steps.  If we have an intersection
C     put the information into FOUND.
C
      FOUND = HAVE
C
C     That's all folks
C
      CALL CHKOUT ( 'PLBORE_2' )
      RETURN
 
      END
